#
# Online Timesheet Product
#
# (C) Copyright 2001 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
from ImageFile import ImageFile
from Globals import DTMLFile

from AccessControl import getSecurityManager
from Persistence import Persistent
from Globals import HTMLFile, HTML
from OFS.Folder import Folder
from Products.ZSQLMethods.SQL import SQL
from DateTime import DateTime

from Client import Client, manage_addClient, manage_addClientForm, manage_editClientForm, manage_editClient
from Project import Project, manage_addProject, manage_addProjectForm, manage_editProjectForm, manage_editProject
from Category import Category, manage_addCategory, manage_addCategoryForm
from TimesheetEntry import TimesheetEntry, MemberTimesheetEntry
from TimeHelpers import getWeek
from SQLHelpers import sqlattr

__internal_version__=0.2

import zLOG
def log(message,summary='',severity=0):
	                        zLOG.LOG('MyDebugLog',severity,summary,message)
				
def manage_addZTimesheet(self, id, title, connection, REQUEST={}):
	""" Add a new Timesheet Manager """
	o = ZTimesheet(id, title, connection)
	self._setObject(id, o)
	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''

manage_addZTimesheetForm=DTMLFile('manage_addZTimesheetForm', globals())

class ZTimesheet(Folder):
	""" An online Timesheet Product """

	meta_type="Timesheet Manager"

	getWeek=getWeek
	manage_addClient=manage_addClient
	manage_addClientForm=manage_addClientForm
	manage_editClientForm=manage_editClientForm
	manage_editClient=manage_editClient

	manage_addClient__roles__=(['Manager'])
	manage_editClient__roles__=(['Manager'])	
	manage_addClientForm__roles__=(['Manager'])
	manage_editClientForm__roles__=(['Manager'])
	
	manage_clientsForm=DTMLFile('manage_clientsForm',globals())
	manage_clientsForm__roles__=(['Manager'])


	manage_addProject=manage_addProject
	manage_addProjectForm=manage_addProjectForm
	manage_editProjectForm=manage_editProjectForm
	manage_editProject=manage_editProject

	manage_addProject__roles__=(['Manager'])
	manage_editProject__roles__=(['Manager'])	
	manage_addProjectForm__roles__=(['Manager'])
	manage_editProjectForm__roles__=(['Manager'])
	
	
	manage_projectsForm=DTMLFile('manage_projectsForm', globals())
	manage_projectsForm__roles__=(['Manager'])

	manage_addCategory=manage_addCategory
	manage_addCategoryForm=manage_addCategoryForm

	manage_addCategory__roles__=(['Manager'])
	manage_addCategoryForm__roles__=(['Manager'])	

	
	manage_categoriesForm=DTMLFile('manage_categoriesForm', globals())
	manage_categoriesForm__roles__=(['Manager'])

	editWeekForm=DTMLFile('editWeekForm', globals())
	manage_categoriesForm__roles__=(['Authenticated'])
	
	index_html=DTMLFile('index_html', globals())
	index_html__roles__=(['Authenticated'])
	
	projectViewForm=DTMLFile('projectViewForm', globals())
	addCommentForm=DTMLFile('addCommentForm', globals())
	viewCommentForm=DTMLFile('viewCommentForm', globals())

	projectViewForm__roles__=(['Authenticated'])
	addCommentForm__roles__=(['Authenticated'])
	viewCommentForm__roles__=(['Authenticated'])	

	meta_types=all_meta_types=((
		{'name':'Project',
		 'action': 'manage_addProjectForm'},
		{'name':'Client',
		 'action': 'manage_addClientForm'},
		{'name':'Category',
		 'action': 'manage_addCategoryForm'},
		))

	manage_options=(
		{'label':'Clients',
		 'action': 'manage_clientsForm'},
		{'label':'Projects',
		 'action': 'manage_projectsForm'},
		{'label':'Categories',
		 'action':'manage_categoriesForm'},
		{'label':'View',
		 'action':'index_html'},
		)

	__internal_version__=0.0 # update this to force sql queries to be updated
	def __init__(self, id, title, connection):
		self.id=id
		self.title=title
		self.connection=connection
		self.addSQLQueries()
		self.version=__internal_version__

	def addSQLQueries(self):
		self.sqlInsertClient=SQL(
			'sqlInsertClient',
			'Insert a New Client',
			self.connection,
			'name address city state postcode country default_bill_rate',
			_sqlInsertClient)

		self.sqlInsertProject=SQL(
			'sqlInsertProject',
			'Insert a New Project',
			self.connection,
			'client_id project_name start_date estimated_end_date estimated_hours project_description isbillable bill_rate',
			_sqlInsertProject)

		self.sqlUpdateClient=SQL(
			'sqlUpdateClient',
			'Update a Client',
			self.connection,
			'client_id name address city state postcode country default_bill_rate',
			_sqlUpdateClient)

		self.sqlUpdateProject=SQL(
			'sqlUpdateProject',
			'Update Project Details',
			self.connection,
			'project_id project_name estimated_end_date estimated_hours project_description isbillable bill_rate',
			_sqlUpdateProject)
		
		self.sqlGetClientIds=SQL(
			'sqlGetClientId',
			'Get all clientIds',
			self.connection,
			'',
			_sqlGetClientIds)

		self.sqlGetOneClient=SQL(
			'sqlGetOneClient',
			'Get One Client',
			self.connection,
			'client_id',
			_sqlGetOneClient)

		self.sqlGetProjectIds=SQL(
			'sqlGetProjectId',
			'Get All Projects',
			self.connection,
			'',
			_sqlGetProjectIds)

		self.sqlGetOneProject=SQL(
			'sqlGetOneProject',
			'Get One Project',
			self.connection,
			'project_id',
			_sqlGetOneProject)

		self.sqlAddMemberToProject=SQL(
			'sqlAddMemberToProject',
			'Add a Member to a Project',
			self.connection,
			'project_id member_id bill_rate is_manager',
			_sqlAddMemberToProject)

		self.sqlRemoveMemberFromProject=SQL(
			'sqlRemoveMemberFromProject',
			'Kill them until they are dead',
			self.connection,
			'project_id member_id',
			_sqlRemoveMemberFromProject)

		self.sqlGetClientProjects=SQL(
			'sqlGetClientProjects',
			'Get all Projects for a client',
			self.connection,
			'client_id',
			_sqlGetClientProjects)

		self.sqlGetProjectMembers=SQL(
			'sqlGetProjectMembers',
			'Get all Members of a Project',
			self.connection,
			'project_id',
			_sqlGetProjectMembers)

		self.sqlGetMembersProjects=SQL(
			'sqlGetMembersProjects',
			'Get all Projects a Member belongs to',
			self.connection,
			'member_id',
			_sqlGetMembersProjects)

		self.sqlGetMembersManagedProjects=SQL(
			'sqlGetMembersManagedProjects',
			'Get all Projects a Member manages',
			self.connection,
			'member_id',
			_sqlGetMembersManagedProjects)

		self.sqlGetTotalHoursForUserForDate=SQL(
			'sqlGetTotalHoursForUserForDate',
			'Get Total Hours for One Day for One User',
			self.connection,
			'member_id date_hours',
			_sqlGetTotalHoursForUserForDate)

		self.sqlGetMemberTimesheetEntries=SQL(
			'sqlGetTimesheetEntries',
			'Get a Weeks worth of hours',
			self.connection,
			'member_id year week',
			_sqlGetMemberTimesheetEntries)

		self.sqlGetMemberProjectTimesheetEntries=SQL(
			'sqlGetTimesheetEntries',
			'Get a Weeks worth of hours',
			self.connection,
			'member_id year week project_id',
			_sqlGetMemberProjectTimesheetEntries)

		self.sqlUpdateTimesheetEntry=SQL(
			'sqlUpdateTimesheetEntry',
			'Update a timesheet entry',
			self.connection,
			'hours hours_id comments',
			_sqlUpdateTimesheetEntry)

		self.sqlGetSpecificMemberTimesheetEntry=SQL(
			'sqlGetSpecificMemberTimesheetEntry',
			'Get a sign line item',
			self.connection,
			'project_id category_id member_id year week dow',
			_sqlGetSpecificMemberTimesheetEntry)

		self.sqlInsertTimesheetEntry=SQL(
			'sqlInsertTimesheetEntry',
			'Insert a new timesheet entry',
			self.connection,
			'project_id client_id member_id hours_date total_hours '+
			'billable comments year week dow status category_id',
			_sqlInsertTimesheetEntry)

		self.sqlGetCategoryIds=SQL(
			'sqlGetCategoryIds',
			'Get Category Ids',
			self.connection,
			'',
			_sqlGetCategoryIds)

		self.sqlGetBillableCategoryIds=SQL(
			'sqlGetBillableCategoryIds',
			'Get Billable Categories',
			self.connection,
			'',
			_sqlGetBillableCategoryIds)

		self.sqlGetNonBillableCategoryIds=SQL(
			'sqlGetNonBillableCategoryIds',
			'Get Non Billable Categories',
			self.connection,
			'',
			_sqlGetNonBillableCategoryIds)

		self.sqlGetOneCategory=SQL(
			'sqlGetOneCategory',
			'Get One Category',
			self.connection,
			'category_id',
			_sqlGetOneCategory)

		self.sqlInsertCategory=SQL(
			'sqlInsertCategory',
			'Insert a new Category',
			self.connection,
			'category_name is_billable',
			_sqlInsertCategory)

		self.sqlGetPendingProjectTimesheetEntries=SQL(
			'sqlGetPendingProjectTimesheetEntries',
			'Get Pending Timesheets',
			self.connection,
			'project_id',
			_sqlGetPendingProjectTimesheetEntries)

		self.sqlGetComments=SQL(
			'sqlGetComments',
			'Get Comment for a timesheet row',
			self.connection,
			'year week project_id category_id member_id',
			_sqlGetComments)

		self.sqlUpdateTimesheetStatus=SQL(
			'sqlUpdateTimesheetStatus',
			'Update the status',
			self.connection,
			'year week project_id category_id member_id status',
			_sqlUpdateTimesheetStatus)

	def __setstate__(self, state):
		Persistent.__setstate__(self, state)
		print self.__internal_version__, self.version
		
		if self.version < self.__internal_version__:
			self.addSQLQueries()
			self.version=self.__internal_version__

	def getTimesheetComments(self, year, week, project_id, category_id,
							 member_id):
		""" returns comments"""
		comments=[]
		r = self.sqlGetComments(year=year, week=week,
								project_id=project_id,
								category_id=category_id,
								member_id=member_id)
		for c in r:
			comments.append(sqlattr(c, 'comments'))

		return comments

	def removeMemberFromProject(self, project_id, username, REQUEST):
		""" Remove someone """
		self.sqlRemoveMemberFromProject(project_id=project_id,
										member_id=username)
		if REQUEST:
			return self.projectViewForm(self, REQUEST)
		return ''

		
	def addMemberToProject(self, project_id, username, bill_rate, isManager=0,
						   REQUEST={}):
		""" Add a New Member to a Project """
		self.sqlAddMemberToProject(project_id=project_id,
								   member_id = username,
								   bill_rate=bill_rate,
								   is_manager=isManager)

		if REQUEST:
			return self.projectViewForm(self, REQUEST)
		return ''

	def getClientFromId(self, client_id): 
		""" Get client """
		result=self.sqlGetOneClient(client_id=client_id)
		if not result:
			return None
		if not len(result):
			return None
		c = result[0]
		id = sqlattr(c, 'client_id')
		name=sqlattr(c, 'client_name')
		def_billRate=sqlattr(c, 'default_bill_rate')
		address_line=sqlattr(c, 'address_line')
		city=sqlattr(c, 'city')
		state=sqlattr(c, 'state')
		postcode=sqlattr(c, 'postcode')
		country=sqlattr(c, 'country')
		cc = Client(id, name, address_line, city, state, postcode, country,
					def_billRate)
		return cc

	def getProjectFromId(self, project_id):
		""" get project """
		result = self.sqlGetOneProject(project_id=int(project_id))
		if not result:
			return None
		if not len(result):
			return None
		p = result[0]
		id = sqlattr(p, 'project_id')
		cid = sqlattr(p, 'client_id')
		name=sqlattr(p, 'project_name')
		start_date=sqlattr(p, 'start_date')
		estimated_end_date=sqlattr(p, 'estimated_end_date')
		actual_end_date=sqlattr(p, 'actual_end_date')
		estimated_hours=sqlattr(p, 'estimated_hours')
		bill_rate=sqlattr(p, 'bill_rate')
		descr=sqlattr(p, 'project_description')
		isbillable=sqlattr(p, 'isbillable')
		status=sqlattr(p, 'status')
		pc = Project(id, cid, name, start_date, estimated_end_date,
					 estimated_hours, bill_rate, descr, isbillable,
					 actual_end_date, status)

		return pc

	def getCategoryFromId(self, category_id):
		""" get category """
		result = self.sqlGetOneCategory(category_id=int(category_id))
		if not result:
			return None
		if not len(result):
			return None
		c = result[0]
		id = sqlattr(c, 'category_id')
		name=sqlattr(c, 'category_name')
		is_billable=sqlattr(c, 'is_billable')

		cc = Category(id, name, is_billable)
		return cc

	def getProjectMembers(self, project_id):
		""" get project members """
		return self.sqlGetProjectMembers(project_id=project_id)

	def getNotProjectMembers(self, project_id):
		""" Get the available users that aren't already members """
		pm=[]
		r = self.sqlGetProjectMembers(project_id=project_id)
		for p in r:
			pm.append(sqlattr(p, 'member_id'))

		zu=self.acl_users.getUserNames()
		nu=[]
		for u in zu:
			if u not in pm:
				nu.append(u)

		return nu
	
	def getCategories(self):
		""" Get all Categories """
		r = self.sqlGetCategoryIds()
		if not r:
			return []
		if not len(r):
			return []

		cs = []
		for c in r:
			cc = self.getCategoryFromId(sqlattr(c, 'category_id'))
			cs.append(cc)
		return cs
		
	def getMemberId(self, member_id):
		""" get a member id """
		if not member_id:
			aun=getSecurityManager().getUser().getUserName()
		else:
			aun = member_id
		return aun

	def getMembersProjects(self, member_id=None):
		""" Get all projects for this member """
		if member_id is None:
			member_id=getSecurityManager().getUser().getUserName()

		aun = self.getMemberId(member_id)
			
		r = self.sqlGetMembersProjects(member_id=aun)
		if not r:
			return []
		if not len(r):
			return []

		prs=[]
		for p in r:
			pp = self.getProjectFromId(sqlattr(p, 'project_id'))
			prs.append(pp)
		return prs

	def getMembersManagedProjects(self, member_id=''):
		""" Get all projects Managed by this member """
		aun = self.getMemberId(member_id)
			
		r = self.sqlGetMembersManagedProjects(member_id=aun)
		if not r:
			return []
		if not len(r):
			return []

		prs=[]
		for p in r:
			pp = self.getProjectFromId(sqlattr(p, 'project_id'))
			prs.append(pp)
		return prs

	def isManager(self, project_id, member_id=''):
		""" Is this member the manager """
		project_id=int(project_id)
		aun = self.getMemberId(member_id)
		r = self.sqlGetMembersManagedProjects(member_id=aun)
		if not r:
			return 0
		if not len(r):
			return 0

		for p in r:
			if project_id == sqlattr(p, 'project_id'):
				return 1
		return 0

	def getYearWeek(self, year, week):
		""" Return a list of DateTime objects """
		return self.getWeek(year, week)

	def getTotalHoursForUserForDate(self, date, member_id=''):
		""" Get all the hours for a user for a single day """
		aun = self.getMemberId(member_id)
			
		x = self.sqlGetTotalHoursForUserForDate(member_id=aun,
												hours_date=date)
		if not x:
			return 0
		if not len(x):
			return 0
		h = x[0]
		hh = sqlattr(h, 'total_hours')
		return hh

	#
	# A week is a list of (7) DateTime objects..
	#
	def isThisWeek(self, week):
		""" Is this week, this week? """
		now = DateTime()
		# day 7 is the start of the day...
		return ( week[0] <= now and now < (week[6]+1))

	def isInThePast(self, week):
		""" Is this week, in the past? """
		now = DateTime()
		return (week[0] < now)

	def isInTheFuture(self, week):
		""" Is this week in the future? """
		now = DateTime()
		# day 7 is the start of the day...
		return (now < (week[6]+1))

	def getMemberTimesheetEntry(self, year, week, member_id=''):
		""" get a timesheet entry (virtual) """
		aun = self.getMemberId(member_id)
		tse=self.sqlGetMemberTimesheetEntries(year=year, week=week,
											  member_id=aun)
		if not tse:
			return None
		ts = TimesheetEntry(tse)
		return ts

	def getProjectOwner(self, project_id):
		""" get owner """
		x = self.getProjectFromId(project_id)
		return x.client_id

	def approveTimesheet(self, year, week, project_id, category_id,
						 member_id, REQUEST={}):
		""" Approve a timesheet """
		self.sqlUpdateTimesheetStatus(member_id=member_id,
									  project_id=project_id,
									  year=year, week=week,
									  category_id=category_id,
									  status=2)
		if REQUEST:
			return self.projectViewForm(self, REQUEST)
		return ''
	
	def updateTimesheetEntry(self, member_id, project_id, year, week, dow,
							 hours, status, category_id, comment):
		""" update timesheet  """
		x= self.sqlGetSpecificMemberTimesheetEntry(member_id=member_id,
												   project_id=project_id,
												   year=year, week=week,
												   category_id=category_id,
												   dow=dow)
		if x and len(x):
			tse = x[0]
			self.sqlUpdateTimesheetEntry(hours_id=sqlattr(tse, 'hours_id'),
										 hours=hours, comments=comment)

			self.sqlUpdateTimesheetStatus(member_id=member_id,
										  project_id=project_id,
										  year=year, week=week,
										  category_id=category_id,
										  status=status)
		else:
			client_id=self.getProjectOwner(project_id)
			w = self.getWeek(year, week)[dow]
			cl = self.getCategoryFromId(category_id)
			
			self.sqlInsertTimesheetEntry(member_id=member_id,
										 client_id=client_id,
										 project_id=project_id,
										 hours_date=w,
										 billable=cl.is_billable,
										 category_id=category_id,
										 comments=comment,
										 year=year, week=week, dow=dow,
										 total_hours=hours, status=status)

	def editWeek(self, year, week, REQUEST, member_id=''):
		""" Edit the Week that was """
		aun = self.getMemberId(member_id)
		status=0
		if REQUEST.has_key('Submit'):
			status=1
		# Iterate over all 20 lines looking for ones that were filled in
		for i in range(1, 21):
			k = 'project_id_%d'%(i)
			ck= 'category_id_%d'%(i)
			if REQUEST.form[k] and REQUEST.form[ck]:
				pid=REQUEST.form[k]
				cid=REQUEST.form[ck]

				# Now iterate over the days...
				for d in range(7):
					# Add or Update a new timesheet entry
					self.updateTimesheetEntry(aun, pid, year, week, d,
											  int(REQUEST.form['dow_%d_%d'%(d, i)]), status, cid,
											  REQUEST.form['comment_%d_%d'%(d, i)])
		REQUEST.RESPONSE.redirect('editWeekForm?year=%d&week=%d'%(year,week))
											  

	def getMemberProjectTimesheetEntries(self, year, week, member_id=''):
		""" member entries """
		mpts=[]
		aun = self.getMemberId(member_id)
		mp = self.getMembersProjects(aun)
		for p in mp:
			tse=self.sqlGetMemberProjectTimesheetEntries(year=year,
														 week=week,
														 member_id=aun,
														 project_id=p.project_id)
			if not tse:
				continue
			if not len(tse):
				continue
			for t in tse:
				ts = MemberTimesheetEntry(t)
				ts.comments=self.getTimesheetComments(year=year, week=week,
													  member_id=aun,
													  project_id=p.project_id,
													  category_id=ts.category_id)
				mpts.append(ts)
		return mpts

	def getPendingProjectTimesheetEntries(self, project_id):
		tse=[]
		r=self.sqlGetPendingProjectTimesheetEntries(project_id=project_id)
		for t in r:
			ts = MemberTimesheetEntry(t)
			ts.comments=self.getTimesheetComments(year=ts.year, week=ts.week,
												  member_id=ts.member_id,
												  project_id=project_id,
												  category_id=ts.category_id)
			tse.append(ts)
			
		return tse

_sqlGetClientIds='''
SELECT client_id FROM clients
'''

_sqlGetOneClient='''
SELECT * FROM clients
WHERE client_id=<dtml-sqlvar client_id type=int>
'''

_sqlGetProjectIds='''
SELECT project_id FROM projects
'''

_sqlGetOneProject='''
SELECT * FROM projects
WHERE project_id=<dtml-sqlvar project_id type=int>
'''

_sqlAddMemberToProject='''
INSERT INTO project_members (project_id, member_id, is_manager, bill_rate)
VALUES (<dtml-sqlvar project_id type=int>,
        <dtml-sqlvar member_id type=string>,
		<dtml-sqlvar bill_rate type=float>,
		<dtml-sqlvar is_manager type=int>)
'''

_sqlRemoveMemberFromProject='''
DELETE FROM project_members
WHERE project_id = <dtml-sqlvar project_id type=int>
AND member_id=<dtml-sqlvar member_id type=string>
'''

_sqlInsertClient='''
INSERT INTO Clients (client_name, address_line, city, state, postcode, country, default_bill_rate)
VALUES (<dtml-sqlvar name type=string>,
        <dtml-sqlvar address type=string>,
		<dtml-sqlvar city type=string>,
		<dtml-sqlvar state type=string>,
		<dtml-sqlvar postcode type=string>,
		<dtml-sqlvar country type=string>,
		<dtml-sqlvar default_bill_rate type=float>)
'''

_sqlUpdateClient='''
UPDATE Clients SET client_name=<dtml-sqlvar name type=string>,
	              address_line=<dtml-sqlvar address type=string>,
				  city=<dtml-sqlvar city type=string>,
				  state=<dtml-sqlvar state type=string>,
				  postcode=<dtml-sqlvar postcode type=string>,
				  country=<dtml-sqlvar country type=string>,
				  default_bill_rate=<dtml-sqlvar default_bill_rate type=float>
WHERE client_id = <dtml-sqlvar client_id type=int>
'''

_sqlInsertProject='''
INSERT INTO projects (client_id, project_name, start_date, estimated_end_date,
                      estimated_hours, bill_rate,
					  project_description, isbillable, status)
VALUES (<dtml-sqlvar client_id type=int>,
        <dtml-sqlvar project_name type=string>,
		<dtml-sqlvar start_date type=string>,
		<dtml-sqlvar estimated_end_date type=string>,
		<dtml-sqlvar estimated_hours type=int>,
		<dtml-sqlvar bill_rate type=float>,
		<dtml-sqlvar project_description type=string>,
		<dtml-sqlvar isbillable type=int>,
		1)
'''

_sqlUpdateProject='''
UPDATE Projects SET project_name=<dtml-sqlvar project_name type=string>,
                    estimated_end_date=<dtml-sqlvar estimated_end_date type=string>,
					estimated_hours=<dtml-sqlvar estimated_hours type=int>,
					project_description=<dtml-sqlvar project_description type=string>,
					isbillable=<dtml-sqlvar isbillable type=int>
WHERE project_id=<dtml-sqlvar project_id type=int>
'''

_sqlGetClientProjects='''
SELECT * FROM Projects
WHERE client_id = <dtml-sqlvar client_id type=int>
'''

_sqlGetProjectMembers='''
SELECT member_id FROM project_members
WHERE project_id = <dtml-sqlvar project_id type=int>
'''

_sqlGetMembersProjects='''
SELECT Projects.project_id FROM Projects, project_members
WHERE Projects.project_id = project_members.project_id
AND project_members.member_id = <dtml-sqlvar member_id type=string>
'''

_sqlGetMembersManagedProjects='''
SELECT Projects.project_id FROM Projects, project_members
WHERE Projects.project_id = project_members.project_id
AND project_members.member_id = <dtml-sqlvar member_id type=string>
AND project_members.is_manager != 0
'''


_sqlGetTotalHoursForUserForDate='''
SELECT SUM(total_hours) AS total_hours FROM hours
WHERE member_id=<dtml-sqlvar member_id type=string>
AND hours_date=<dtml-sqlvar hours_date type=string>
'''

#
# Avoid date comparisons which can suck across different
# databases.
#
_sqlGetMemberTimesheetEntries='''
SELECT SUM(total_hours) AS total_hours, status, dow, category_id, project_id
FROM hours
WHERE member_id=<dtml-sqlvar member_id type=string>
AND year = <dtml-sqlvar year type=int>
AND week = <dtml-sqlvar week type=int>
GROUP BY project_id, category_id,status, dow
'''


_sqlGetMemberProjectTimesheetEntries='''
SELECT *
FROM timesheet
WHERE member_id=<dtml-sqlvar member_id type=string>
AND year = <dtml-sqlvar year type=int>
AND week = <dtml-sqlvar week type=int>
AND project_id = <dtml-sqlvar project_id type=int>
ORDER BY project_id, category_id
'''

_sqlGetPendingProjectTimesheetEntries='''
SELECT *
FROM timesheet
WHERE project_id=<dtml-sqlvar project_id type=int>
AND status = 1
ORDER by member_id, week
'''

_sqlGetSpecificMemberTimesheetEntry='''
SELECT * FROM hours
WHERE member_id=<dtml-sqlvar member_id type=string>
AND year = <dtml-sqlvar year type=int>
AND week = <dtml-sqlvar week type=int>
AND dow = <dtml-sqlvar dow type=int>
AND project_id = <dtml-sqlvar project_id type=int>
AND category_id = <dtml-sqlvar category_id type=int>
ORDER BY dow
'''

_sqlUpdateTimesheetEntry='''
UPDATE hours
SET total_hours=<dtml-sqlvar hours type=float>,
    date_entered=NOW(),
	comments=<dtml-sqlvar comments type=string>
WHERE hours_id=<dtml-sqlvar hours_id type=int>
'''

_sqlInsertTimesheetEntry='''
INSERT INTO HOURS (project_id, client_id, member_id, hours_date,
                   date_entered, total_hours, billable, category_id,
				   comments, year, week, dow, status)
VALUES (<dtml-sqlvar project_id type=int>,
        <dtml-sqlvar client_id type=int>,
		<dtml-sqlvar member_id type=string>,
		<dtml-sqlvar hours_date type=string>,
		NOW(),
		<dtml-sqlvar total_hours type=float>,
		<dtml-sqlvar billable type=int>,
		<dtml-sqlvar category_id type=int>,
		<dtml-sqlvar comments type=string>,
		<dtml-sqlvar year type=int>,
		<dtml-sqlvar week type=int>,
		<dtml-sqlvar dow type=int>,
		<dtml-sqlvar status type=int>)
'''

_sqlGetCategoryIds='''
SELECT category_id from Categories
'''

_sqlGetBillableCategoryIds='''
SELECT category_id from Categories
WHERE is_billable != 0
'''

_sqlGetNonBillableCategoryIds='''
SELECT category_id from Categories
WHERE is_billable = 0
'''


_sqlGetOneCategory='''
SELECT * FROM Categories
WHERE category_id=<dtml-sqlvar category_id type=int>
'''

_sqlInsertCategory='''
INSERT INTO Categories (category_name, is_billable)
VALUES (<dtml-sqlvar category_name type=string>,
        <dtml-sqlvar is_billable type=int> )
'''

_sqlGetComments='''
SELECT comments FROM hours
WHERE year=<dtml-sqlvar year type=int>
AND   week=<dtml-sqlvar week type=int>
AND project_id=<dtml-sqlvar project_id type=int>
AND category_id=<dtml-sqlvar category_id type=int>
AND member_id=<dtml-sqlvar member_id type=string>
ORDER BY dow ASC
'''

_sqlUpdateTimesheetStatus='''
UPDATE HOURS set status = <dtml-sqlvar status type=int>
WHERE year=<dtml-sqlvar year type=int>
AND   week=<dtml-sqlvar week type=int>
AND project_id=<dtml-sqlvar project_id type=int>
AND category_id=<dtml-sqlvar category_id type=int>
AND member_id=<dtml-sqlvar member_id type=string>
AND status < <dtml-sqlvar status type=int>
'''
